// lib.h

#ifndef __LIB_H__
#define __LIB_H__

#define MAX_DMA_CHAIN 8

#define ADR_SRAM     (u8*)0xE000000
#define ADR_EWRAM    (u8*)&__ewram_overlay_lma;

#define IN_IWRAM __attribute__ ((section (".iwram")))
#define IN_EWRAM __attribute__ ((section (".ewram")))
#define IN_IWRAM_NAKED __attribute__ ((section (".iwram"), long_call, naked))
#define IN_EWRAM_NAKED __attribute__ ((section (".ewram"), long_call, naked))



//BG
void BGSetPal16(u16 No, u16 *pal);
void BGSetPal256(u16 *pal);
void BGSetPalMap(u8 No,u32 size, u16 pal);
void BGSetChrData(u8 No,u16 *img,u32 size);
void BGSetMapData(u8 No,u16 *img,u32 size);
void BGDrawStr(u8 BG,u8 x,u8 y,u8* str);
void BGDrawChar(u8 No, u8 x,u8 y,u16 palno,u16 chr);
void BGDrawCls(u8 BG, u16 chr);
void BGDrawBox(u8 BG, u16 x, u16 y, u16 stopX, u16 stopY, u16 chr);
u16  BGGetChar(u8 No, u8 x, u8 y);


//Mode3
u16  Mode3GetDotRGB(u8 ,u8);
void Mode3DrawCls(u16);
void Mode3DrawDot(u8, u8, u16);
void Mode3DrawLine(u8, u8, u8, u8, u16);
void Mode3DrawBox(u8, u8, u8, u8, u16);
void Mode3DrawCircle(u16, u16, u16, u16);
void Mode3DrawFillCircle(u16, u16, u16, u16);
void Mode3DrawImage(u8 x, u8 y, u8 cx,u8 cy,u16* image);
//Mode3 Font BitMap
void Mode3SetBitMapFont(u8 *offset, u16 cx, u16 cy);
void Mode3DrawBitMapStr(u8 x,u8 y, u8* str);
void Mode3DrawBitMapChar(u8 x,u8 y,u16 chr);
//Mode3 Font SJIS
void Mode3SetSJISFont(u8* sheet, u8* offset, u16 cnt, u8 cx, u8 cy);
void Mode3DrawSJISStr(u8 x,u8 y,u8* texts,u16 color);
void Mode3DrawSJISChar(u8 x, u8 y, u16 num, u16 col);
void Mode3DrawSJIS(u8 x, u8 y, u8* chr, u16 col); //2Bytep
void Mode3DrawSJISAscii(u8 x, u8 y, u16 num, u16 col);
void Mode3DrawSJISBlock(u8 font,int x1,int y1,u16 color);
//Mode3 Effect
//void Mode3DrawFadeOut();
//void Mode3DrawFadeIn();



//Sprite
void SpriteInit();
void SpriteSetPal16(u16 *pal);
void SpriteSetPal16No(u16 num, u16 palno);
void SpriteSetPal256(u16 *pal);
void SpriteSetData(u16 *img,u32 size);
void SpriteSetSize(u16 num,u16 size,u16 form,u16 color);
void SpriteSetChr(u16 num, u16 ch);
void SpriteSetPriority(u16 num, u16 priority);
void SpriteSetVertical(u16 num, u8 flag);
void SpriteSetRotate(u16 num, u8 angle, u16 Scale);
void SpriteSetRotateFlag(u16 num, u16 no, u16 double_flag);
void SpriteSetScale(u16 num, u16 xscale, u16 yscale);
void SpriteMove(u16 num, s16 x, s16 y);


//Sound
void SoundInit();
void SoundSetData(s8 *offset, u32 size);
void SoundStart();
void SoundStop();



//L[
void KeyInit();
u16  KeyInput();



//fobN
void dprint(u8* sz);
int  dprintf(char *fmt, ...);
void DebugMessage(u8* msg);
//void DebufAssert(hoge);


//DMA]
void DMA3MemcpyFIX16(u32, u32, u16);
void DMA3MemcpyINC16(u32, u32, u16);
void DMA3MemcpyINC32(u32, u32, u16);



//DMAChain
void DMAChainInit();
void DMAChainEnqueue(u32 src, u32 dst, u16 size, u16 mode);
void DMAChainTransfer();


//
void *_malloc(u32 size);
void _mfree(void *ptr);



//EFCg
void WaitKey(u16);



//̑
void  WaitForVsync();
char* bprintf(char *p, char *fmt, ...);

f32   _Sin(u8);
f32   _Cos(u8);
u8    GetAngle(s16 dx, s16 dy);

u8 HitChkBox(u16 srcX, u16 srcY, u16 srcCx, u16 srcCy, u16 dstX, u16 dstY, u16 dstCx,u16 dstCy);
u8 HitChkCircle(u16 srcX, u16 srcY, u16 srcR, u16 dstX, u16 dstY, u16 dstR);



// ExtendTinyFileSystem
u8*  EtfsOffset(u32 data_offset);
//u8*  Etfsfopen(u32 offset, u32 size);
u8*  EtfsfopenLZ77(u32 offset, u32 size);
u8   EtfsReadu8(u32);
u16  EtfsReadu16(u32);
u32  EtfsReadu32(u32);



//Seek
void SeekSetChr(u8* pOffset, u32 size, u8 chr);
u8*  SeekNext();
u8*  SeekPrev();
u8   SeekHasMore();



//SRAM
u8   SRAMRead8 (u32);
u16  SRAMRead16(u32);
u32  SRAMRead32(u32);
void SRAMWrite8 (u32,u8);
void SRAMWrite16(u32,u16);
void SRAMWrite32(u32,u32);
//u8 SRAMComp8 (u32,u8);
//u8 SRAMComp16(u32,u16);
//u8 SRAMComp32(u32,u32);

typedef struct{
	u16* offset;
	u16  cx;
	u16  cy;
	u16  size;
} ST_FONTBITMAP;

typedef struct{
	u16* FontSheet;
	u8*  Offset;
	u16  cx;
	u16  cy;
	u16  size;
	u16  cnt;
} ST_FONTSJIS;

typedef struct{
	s8  *Offset;
	u32 size;
} ST_WAVE;

typedef struct{
	u16 cur;
	u16 old;
	u16 trg;
	u16 off;
} ST_KEY;

typedef struct{
	u8* Now;
	u8* Start;
	u8* End;
	u8  Chr;
} ST_SEEK;

typedef struct {
	u32 src;
	u32 dst;
	u16 size;
	u16 mode;
} ST_DMA_CHAIN;

#endif
